package dk.apaq.crud.core;

import dk.apaq.crud.CrudListSpecification;
import dk.apaq.filter.Filter;
import dk.apaq.filter.FiltrationItem;
import dk.apaq.filter.core.BeanFiltrationItem;
import dk.apaq.filter.limit.Limit;
import dk.apaq.filter.sort.Sorter;
import java.util.*;

/**
 *
 */
public class CollectionCrud<BEANTYPE> extends BaseCompleteCrud<String, BEANTYPE> {

    private final List<BEANTYPE> beans;
    private final IdResolver<BEANTYPE> idResolver;

    private static class DefaultIdResolver<BEANTYPE> implements IdResolver<BEANTYPE> {

        private final WeakHashMap<BEANTYPE, String> idmap = new WeakHashMap<BEANTYPE, String>();

        public String getIdForBean(BEANTYPE bean) {
            String id = idmap.get(bean);
            if (id == null) {
                id = UUID.randomUUID().toString();
                idmap.put(bean, id);
            }
            return id;
        }
    }

    public interface IdResolver<BEANTYPE> {

        String getIdForBean(BEANTYPE bean);
    }

    public CollectionCrud() {
        this(null, new DefaultIdResolver<BEANTYPE>());
    }
    
    public CollectionCrud(IdResolver<BEANTYPE> idResolver) {
        this(null, idResolver);
    }
    

    public CollectionCrud(List<BEANTYPE> beans) {
        this(beans, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionCrud(List<BEANTYPE> beans, IdResolver<BEANTYPE> idResolver) {
        this.beans = beans == null ? new ArrayList<BEANTYPE>() : beans;
        this.idResolver = idResolver;
    }

    public String create() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public <T extends BEANTYPE> String create(T entity) {
        beans.add(entity);
        return idResolver.getIdForBean(entity);
    }

    public <T extends BEANTYPE> T createAndRead(T entity) {
        beans.add(entity);
        return entity;
    }

    public BEANTYPE update(BEANTYPE entity) {
        return entity;
    }

    public void delete(String id) {
        fireOnBeforeDelete(id);
        
        Iterator<BEANTYPE> it = beans.iterator();
        
        while(it.hasNext()) {
            BEANTYPE b = it.next();
            if(id.equals(idResolver.getIdForBean(b))) {
                it.remove();
                break;
            }
        }
        
        fireOnDelete(id);
    }
    
    public BEANTYPE read(String id) {
        fireOnBeforeRead(id);
        for (BEANTYPE current : beans) {
            String currentId = idResolver.getIdForBean(current);
            if (id.equals(currentId)) {
                fireOnRead(id, current);
                return current;
            }
        }
        return null;
    }

    public List<String> listIds(Filter filter, Sorter sorter, Limit limit) {
        CrudListSpecification listSpec = new CrudListSpecification(filter, sorter, limit);
        fireOnBeforeList(listSpec);
        
        List<BEANTYPE> list = new ArrayList<BEANTYPE>(beans);
        list = listSpec.getFilter() == null ? list : filterList(list, listSpec.getFilter());
        list = listSpec.getSorter() == null ? list : sortList(list, listSpec.getSorter());

        List<String> idlist = new ArrayList<String>();

        int count = 0;
        for (BEANTYPE e : list) {
            if (limit == null | (limit != null && count >= limit.getOffset() && count < (limit.getOffset() + limit.getCount()))) {
                String id = idResolver.getIdForBean(e);
                idlist.add(id);
            }
            count++;
        }
        return idlist;
    }
    
    public List<BEANTYPE> list(Filter filter, Sorter sorter, Limit limit) {
        List<String> idlist = listIds(filter, sorter, limit);
        List<BEANTYPE> beanlist = new ArrayList<BEANTYPE>();
        
        for(String id : idlist) {
            beanlist.add(read(id));
        }
        return beanlist;
        
    }


    private List<BEANTYPE> filterList(List<BEANTYPE> list, Filter filter) {
        List<BEANTYPE> filtered = new ArrayList();
        for (BEANTYPE e : list) {
            FiltrationItem fi = new BeanFiltrationItem(e);
            if (filter.passesFilter(fi)) {
                filtered.add(e);
            }
        }
        return filtered;
    }

    private List<BEANTYPE> sortList(List<BEANTYPE> list, final Sorter sorter) {
        List<BeanFiltrationItem<BEANTYPE>> sortList = new ArrayList<BeanFiltrationItem<BEANTYPE>>();
        List<BEANTYPE> sorted = new ArrayList<BEANTYPE>();

        for (BEANTYPE current : list) {
            sortList.add(new BeanFiltrationItem<BEANTYPE>(current));
        }

        Collections.sort(sortList, new Comparator<BeanFiltrationItem>() {

            @Override
            public int compare(BeanFiltrationItem o1, BeanFiltrationItem o2) {
                if (sorter == null) {
                    return 0;
                }

                switch (sorter.compare(o1, o2)) {
                    case GreaterThan:
                        return 1;
                    case LessThan:
                        return -1;
                    default:
                        return 0;
                }
            }
        });

        for (BeanFiltrationItem<BEANTYPE> current : sortList) {
            sorted.add(current.getBean());
        }
        return sorted;
    }
    
    
}
